@@ -1,7 +1,9 @@
+local S = minetest.get_translator("pipeworks")
+
 local straight = function(pos, node, velocity, stack) return {velocity} end
 
 minetest.register_node("pipeworks:steel_block_embedded_tube", {
-	description = "Airtight steelblock embedded tube",
+	description = S("Airtight steelblock embedded tube"),
 	tiles = {
 		"default_steel_block.png", "default_steel_block.png",
 		"default_steel_block.png", "default_steel_block.png",
@@ -26,6 +28,7 @@ minetest.register_node("pipeworks:steel_block_embedded_tube", {
 	after_dig_node = pipeworks.after_dig,
 	on_rotate = pipeworks.on_rotate,
 })
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:steel_block_embedded_tube"
 
 minetest.register_craft( {
 	output = "pipeworks:steel_block_embedded_tube 1",
@@ -43,9 +46,13 @@ local pane_box = {
 		{ -8/16, -8/16, -1/16, 8/16, 8/16, 1/16 } -- pane
 	}
 }
+
+local texture_alpha_mode = minetest.features.use_texture_alpha_string_modes
+	and "clip" or true
+
 minetest.register_node("pipeworks:steel_pane_embedded_tube", {
 	drawtype = "nodebox",
-	description = "Airtight panel embedded tube ",
+	description = S("Airtight panel embedded tube"),
 	tiles = {
 		"pipeworks_pane_embedded_tube_sides.png^[transformR90",
 		"pipeworks_pane_embedded_tube_sides.png^[transformR90",
@@ -53,6 +60,7 @@ minetest.register_node("pipeworks:steel_pane_embedded_tube", {
 		"pipeworks_pane_embedded_tube_sides.png",
 		"pipeworks_pane_embedded_tube_ends.png", "pipeworks_pane_embedded_tube_ends.png",
 		},
+	use_texture_alpha = texture_alpha_mode,
 	node_box = pane_box,
 	selection_box = pane_box,
 	collision_box = pane_box,
@@ -74,6 +82,7 @@ minetest.register_node("pipeworks:steel_pane_embedded_tube", {
 	after_dig_node = pipeworks.after_dig,
 	on_rotate = pipeworks.on_rotate,
 })
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:steel_pane_embedded_tube"
 
 minetest.register_craft( {
 	output = "pipeworks:steel_pane_embedded_tube 1",
