@@ -10,7 +10,7 @@ function pipeworks.fix_after_rotation(pos, node, user, mode, new_param2)
 
 	if string.find(node.name, "spigot") then new_param2 = new_param2 % 4 end
 
-	newnode = string.gsub(node.name, "_on", "_off")
+	local newnode = string.gsub(node.name, "_on", "_off")
 	minetest.swap_node(pos, { name = newnode, param2 = new_param2 })
 	pipeworks.scan_for_pipe_objects(pos)
 
@@ -20,7 +20,7 @@ end
 function pipeworks.rotate_on_place(itemstack, placer, pointed_thing)
 
 	local playername = placer:get_player_name()
-	if not minetest.is_protected(pointed_thing.under, playername) 
+	if not minetest.is_protected(pointed_thing.under, playername)
 	   and not minetest.is_protected(pointed_thing.above, playername) then
 
 		local node = minetest.get_node(pointed_thing.under)
@@ -36,9 +36,7 @@ function pipeworks.rotate_on_place(itemstack, placer, pointed_thing)
 			local under = pointed_thing.under
 			local fdir = minetest.dir_to_facedir(placer:get_look_dir())
 			local undernode = minetest.get_node(under)
-			local abovenode = minetest.get_node(above)
 			local uname = undernode.name
-			local aname = abovenode.name
 			local isabove = (above.x == under.x) and (above.z == under.z) and (pitch > 0)
 			local pos1 = above
 
@@ -105,6 +103,7 @@ if minetest.get_modpath("mesecons") then
 	}
 end
 
+--[[
 local pipes_devicelist = {
 	"pump",
 	"valve",
@@ -120,14 +119,15 @@ local pipes_devicelist = {
 	"storage_tank_9",
 	"storage_tank_10"
 }
+--]]
 
 -- Now define the nodes.
 
 local states = { "on", "off" }
-local dgroups = ""
 
 for s in ipairs(states) do
 
+	local dgroups
 	if states[s] == "off" then
 		dgroups = {snappy=3, pipe=1}
 	else
@@ -187,11 +187,11 @@ for s in ipairs(states) do
 		paramtype = "light",
 		paramtype2 = "facedir",
 		selection_box = {
-	             	type = "fixed",
+			type = "fixed",
 			fixed = { -5/16, -4/16, -8/16, 5/16, 5/16, 8/16 }
 		},
 		collision_box = {
-	             	type = "fixed",
+			type = "fixed",
 			fixed = { -5/16, -4/16, -8/16, 5/16, 5/16, 8/16 }
 		},
 		groups = dgroups,
@@ -222,6 +222,8 @@ for s in ipairs(states) do
 		new_flow_logic_register.directional_horizonal_rotate(nodename_valve_empty, true)
 	end
 end
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:pump_off"
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:valve_off_empty"
 
 local nodename_valve_loaded = "pipeworks:valve_on_loaded"
 minetest.register_node(nodename_valve_loaded, {
@@ -233,11 +235,11 @@ minetest.register_node(nodename_valve_loaded, {
 	paramtype = "light",
 	paramtype2 = "facedir",
 	selection_box = {
-             	type = "fixed",
+		type = "fixed",
 		fixed = { -5/16, -4/16, -8/16, 5/16, 5/16, 8/16 }
 	},
 	collision_box = {
-             	type = "fixed",
+		type = "fixed",
 		fixed = { -5/16, -4/16, -8/16, 5/16, 5/16, 8/16 }
 	},
 	groups = {snappy=3, pipe=1, not_in_creative_inventory=1},
@@ -301,6 +303,7 @@ minetest.register_node("pipeworks:grating", {
 	end,
 	on_rotate = false
 })
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:grating"
 
 -- outlet spigot
 
@@ -378,6 +381,8 @@ minetest.register_node(nodename_spigot_loaded, {
 	drop = "pipeworks:spigot",
 	on_rotate = pipeworks.fix_after_rotation
 })
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:spigot"
+
 -- new flow logic does not currently distinguish between these two visual states.
 -- register both so existing flowing spigots continue to work (even if the visual doesn't match the spigot's behaviour).
 new_flow_logic_register.directional_horizonal_rotate(nodename_spigot_empty, false)
@@ -441,6 +446,9 @@ minetest.register_node(nodename_panel_loaded, {
 	drop = "pipeworks:entry_panel_empty",
 	on_rotate = pipeworks.fix_after_rotation
 })
+
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:entry_panel_empty"
+
 -- TODO: AFAIK the two panels have no visual difference, so are redundant under new flow logic - alias?
 new_flow_logic_register.directional_horizonal_rotate(nodename_panel_empty, true)
 new_flow_logic_register.directional_horizonal_rotate(nodename_panel_loaded, true)
@@ -465,7 +473,7 @@ minetest.register_node(nodename_sensor_empty, {
 	end,
 	on_construct = function(pos)
 		if mesecon then
-			mesecon.receptor_off(pos, rules) 
+			mesecon.receptor_off(pos, pipeworks.mesecons_rules)
 		end
 	end,
 	selection_box = {
@@ -504,7 +512,7 @@ minetest.register_node(nodename_sensor_loaded, {
 	end,
 	on_construct = function(pos)
 		if mesecon then
-			mesecon.receptor_on(pos, rules) 
+			mesecon.receptor_on(pos, pipeworks.mesecons_rules)
 		end
 	end,
 	selection_box = {
@@ -525,6 +533,8 @@ minetest.register_node(nodename_sensor_loaded, {
 	mesecons = pipereceptor_on,
 	on_rotate = pipeworks.fix_after_rotation
 })
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:flow_sensor_empty"
+
 new_flow_logic_register.directional_horizonal_rotate(nodename_sensor_empty, true)
 new_flow_logic_register.directional_horizonal_rotate(nodename_sensor_loaded, true)
 -- activate flow sensor at roughly half the pressure pumps drive pipes
@@ -603,6 +613,7 @@ for fill = 0, 10 do
 		on_rotate = false
 	})
 end
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:storage_tank_0"
 
 -- fountainhead
 
@@ -626,7 +637,7 @@ minetest.register_node(nodename_fountain_empty, {
 	end,
 	on_construct = function(pos)
 		if mesecon then
-			mesecon.receptor_on(pos, rules) 
+			mesecon.receptor_on(pos, pipeworks.mesecons_rules)
 		end
 	end,
 	selection_box = {
@@ -639,6 +650,7 @@ minetest.register_node(nodename_fountain_empty, {
 	},
 	on_rotate = false
 })
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:fountainhead"
 
 local nodename_fountain_loaded = "pipeworks:fountainhead_pouring"
 minetest.register_node(nodename_fountain_loaded, {
@@ -661,7 +673,7 @@ minetest.register_node(nodename_fountain_loaded, {
 	end,
 	on_construct = function(pos)
 		if mesecon then
-			mesecon.receptor_on(pos, rules) 
+			mesecon.receptor_on(pos, pipeworks.mesecons_rules)
 		end
 	end,
 	selection_box = {
@@ -734,6 +746,7 @@ minetest.register_node(nodename_sp_loaded, {
 	check_for_pole = pipeworks.check_for_vert_pipe,
 	check_for_horiz_pole = pipeworks.check_for_horiz_pipe
 })
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:straight_pipe_empty"
 
 new_flow_logic_register.directional_horizonal_rotate(nodename_sp_empty, true)
 new_flow_logic_register.directional_horizonal_rotate(nodename_sp_loaded, true)
