@@ -5,8 +5,7 @@
 --
 
 pipeworks = {}
-
-local DEBUG = false
+pipeworks.ui_cat_tube_list = {}
 
 pipeworks.worldpath = minetest.get_worldpath()
 pipeworks.modpath = minetest.get_modpath("pipeworks")
@@ -71,8 +70,8 @@ function pipeworks.may_configure(pos, player)
 	local meta = minetest.get_meta(pos)
 	local owner = meta:get_string("owner")
 
-	if owner ~= "" then -- wielders and filters
-		return owner == name
+	if owner ~= "" and owner == name then -- wielders and filters
+		return true
 	end
 	return not minetest.is_protected(pos, name)
 end
@@ -150,4 +149,16 @@ end
 
 minetest.register_alias("pipeworks:pipe", "pipeworks:pipe_110000_empty")
 
-print("Pipeworks loaded!")
+-- Unified Inventory categories integration
+
+if unified_inventory and unified_inventory.registered_categories then
+	if not unified_inventory.registered_categories["automation"] then
+		unified_inventory.register_category("automation", {
+			symbol = "pipeworks:lua_tube000000",
+			label = "Automation components"
+		})
+	end
+	unified_inventory.add_category_items("automation", pipeworks.ui_cat_tube_list)
+end
+
+minetest.log("info", "Pipeworks loaded!")
