@@ -11,11 +11,11 @@ local new_flow_logic_register = pipeworks.flowables.register
 local polys = ""
 if pipeworks.enable_lowpoly then polys = "_lowpoly" end
 
-local vti = {4, 3, 2, 1, 6, 5}
+--~ local vti = {4, 3, 2, 1, 6, 5}
 local cconnects = {{}, {1}, {1, 2}, {1, 3}, {1, 3, 5}, {1, 2, 3}, {1, 2, 3, 5}, {1, 2, 3, 4}, {1, 2, 3, 4, 5}, {1, 2, 3, 4, 5, 6}}
 for index, connects in ipairs(cconnects) do
 	local outsel = {}
-	
+
 	local jx = 0
 	local jy = 0
 	local jz = 0
@@ -30,11 +30,13 @@ for index, connects in ipairs(cconnects) do
 		table.insert(outsel, pipeworks.pipe_selectboxes[v])
 	end
 
+	--[[
 	if #connects == 1 then
 		local v = connects[1]
 		v = v-1 + 2*(v%2) -- Opposite side
 	end
-	
+	--]]
+
 	local pgroups = {snappy = 3, pipe = 1, not_in_creative_inventory = 1}
 	local pipedesc = S("Pipe Segment").." "..dump(connects)
 
@@ -42,11 +44,11 @@ for index, connects in ipairs(cconnects) do
 		pgroups = {snappy = 3, tube = 1}
 		pipedesc = S("Pipe Segment")
 	end
-	
+
 	local outimg_e = { "pipeworks_pipe_plain.png" }
 	local outimg_l = { "pipeworks_pipe_plain.png" }
 
-	if index == 3 then 
+	if index == 3 then
 		outimg_e = { "pipeworks_pipe_3_empty.png" }
 		outimg_l = { "pipeworks_pipe_3_loaded.png" }
 	end
@@ -88,7 +90,7 @@ for index, connects in ipairs(cconnects) do
 		check_for_horiz_pole = pipeworks.check_for_horiz_pipe,
 		pipenumber = index
 	})
-	
+
 	local pgroups = {snappy = 3, pipe = 1, not_in_creative_inventory = 1}
 
 	minetest.register_node("pipeworks:pipe_"..index.."_loaded", {
@@ -123,7 +125,7 @@ for index, connects in ipairs(cconnects) do
 		check_for_horiz_pole = pipeworks.check_for_horiz_pipe,
 		pipenumber = index
 	})
-	
+
 	local emptypipe = "pipeworks:pipe_"..index.."_empty"
 	local fullpipe = "pipeworks:pipe_"..index.."_loaded"
 	table.insert(pipes_empty_nodenames, emptypipe)
@@ -131,7 +133,7 @@ for index, connects in ipairs(cconnects) do
 	new_flow_logic_register.simple(emptypipe)
 	new_flow_logic_register.simple(fullpipe)
 end
-
+pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:pipe_1_empty"
 
 
 if REGISTER_COMPATIBILITY then
@@ -239,7 +241,7 @@ minetest.register_abm({
 	nodenames = {"pipeworks:spigot","pipeworks:spigot_pouring"},
 	interval = 1,
 	chance = 1,
-	action = function(pos, node, active_object_count, active_object_count_wider) 
+	action = function(pos, node, active_object_count, active_object_count_wider)
 		pipeworks.spigot_check(pos,node)
 	end
 })
@@ -248,7 +250,7 @@ minetest.register_abm({
 	nodenames = {"pipeworks:fountainhead","pipeworks:fountainhead_pouring"},
 	interval = 1,
 	chance = 1,
-	action = function(pos, node, active_object_count, active_object_count_wider) 
+	action = function(pos, node, active_object_count, active_object_count_wider)
 		pipeworks.fountainhead_check(pos,node)
 	end
 })
