@@ -1,6 +1,6 @@
-
+local S = minetest.get_translator("pipeworks")
 -- the default tube and default textures
-pipeworks.register_tube("pipeworks:tube", "Pneumatic tube segment")
+pipeworks.register_tube("pipeworks:tube", S("Pneumatic tube segment"))
 minetest.register_craft( {
 	output = "pipeworks:tube_1 6",
 	recipe = {
@@ -10,10 +10,39 @@ minetest.register_craft( {
 	},
 })
 
+-- The hammers that can be used to break/repair tubes
+local allowed_hammers = {
+	"anvil:hammer",
+	"cottages:hammer",
+	"glooptest:hammer_steel",
+	"glooptest:hammer_bronze",
+	"glooptest:hammer_diamond",
+	"glooptest:hammer_mese",
+	"glooptest:hammer_alatro",
+	"glooptest:hammer_arol"
+}
+
+-- Convert the above list to a format that's easier to look up
+for _,hammer in ipairs(allowed_hammers) do
+	allowed_hammers[hammer] = true
+end
+
+-- Check if the player is holding a suitable hammer or not - if they are, apply wear to it
+function pipeworks.check_and_wear_hammer(player)
+	local itemstack = player:get_wielded_item()
+	local wieldname = itemstack:get_name()
+	if allowed_hammers[wieldname] then
+		itemstack:add_wear(1000)
+		player:set_wielded_item(itemstack)
+		return true
+	end
+	return false
+end
+
 local nodecolor = 0xffff3030
 
 pipeworks.register_tube("pipeworks:broken_tube", {
-	description = "Broken Tube (you hacker you)",
+	description = S("Broken Tube"),
 	plain = { { name = "pipeworks_broken_tube_plain.png", backface_culling = false, color = nodecolor } },
 	noctr = { { name = "pipeworks_broken_tube_plain.png", backface_culling = false, color = nodecolor } },
 	ends  = { { name = "pipeworks_broken_tube_end.png",   color = nodecolor } },
@@ -35,30 +64,28 @@ pipeworks.register_tube("pipeworks:broken_tube", {
 			local itemstack = puncher:get_wielded_item()
 			local wieldname = itemstack:get_name()
 			local playername = puncher:get_player_name()
-			print("[Pipeworks] "..playername.." struck a broken tube at "..minetest.pos_to_string(pos))
-			if   wieldname == "anvil:hammer"
-			  or wieldname == "cottages:hammer"
-			  or wieldname == "glooptest:hammer_steel"
-			  or wieldname == "glooptest:hammer_bronze"
-			  or wieldname == "glooptest:hammer_diamond"
-			  or wieldname == "glooptest:hammer_mese"
-			  or wieldname == "glooptest:hammer_alatro"
-			  or wieldname == "glooptest:hammer_arol" then
-				local meta = minetest.get_meta(pos)
-				local was_node = minetest.deserialize(meta:get_string("the_tube_was"))
-				if was_node and was_node ~= "" then
-					print("            with "..wieldname.." to repair it.")
-					minetest.swap_node(pos, { name = was_node.name, param2 = was_node.param2 })
-					pipeworks.scan_for_tube_objects(pos)
-					itemstack:add_wear(1000)
-					puncher:set_wielded_item(itemstack)
-					return itemstack
-				else
-					print("            but it can't be repaired.")
-				end
-			else
-				print("            with "..wieldname.." but that tool is too weak.")
+			local log_msg = playername.." struck a broken tube at "..minetest.pos_to_string(pos).."\n"
+			local meta = minetest.get_meta(pos)
+			local was_node = minetest.deserialize(meta:get_string("the_tube_was"))
+			if not was_node then
+				pipeworks.logger(log_msg.."            but it can't be repaired.")
+				return
 			end
+			if not pipeworks.check_and_wear_hammer(puncher) then
+				if wieldname == "" then
+					pipeworks.logger(log_msg.."            by hand. It's not very effective.")
+					if minetest.settings:get_bool("enable_damage") then
+						minetest.chat_send_player(playername,S("Broken tubes may be a bit sharp. Perhaps try with a hammer?"))
+						puncher:set_hp(puncher:get_hp()-1)
+					end
+				else
+					pipeworks.logger(log_msg.."            with "..wieldname.." but that tool is too weak.")
+				end
+				return
+			end
+			pipeworks.logger(log_msg.."            with "..wieldname.." to repair it.")
+			minetest.swap_node(pos, { name = was_node.name, param2 = was_node.param2 })
+			pipeworks.scan_for_tube_objects(pos)
 		end
 	}
 })
@@ -69,7 +96,7 @@ pipeworks.register_tube("pipeworks:broken_tube", {
 if pipeworks.enable_priority_tube then
 	local color = "#ff3030:128"
 	pipeworks.register_tube("pipeworks:priority_tube", {
-			description = "High Priority Tube Segment",
+			description = S("High Priority Tube Segment"),
 			inventory_image = "pipeworks_tube_inv.png^[colorize:" .. color,
 			plain = { { name = "pipeworks_tube_plain.png", color = nodecolor } },
 			noctr = { { name = "pipeworks_tube_noctr.png", color = nodecolor } },
@@ -91,7 +118,7 @@ end
 
 if pipeworks.enable_accelerator_tube then
 	pipeworks.register_tube("pipeworks:accelerator_tube", {
-			description = "Accelerating Pneumatic Tube Segment",
+			description = S("Accelerating Pneumatic Tube Segment"),
 			inventory_image = "pipeworks_accelerator_tube_inv.png",
 			plain = { "pipeworks_accelerator_tube_plain.png" },
 			noctr = { "pipeworks_accelerator_tube_noctr.png" },
@@ -116,7 +143,7 @@ end
 
 if pipeworks.enable_crossing_tube then
 	pipeworks.register_tube("pipeworks:crossing_tube", {
-			description = "Crossing Pneumatic Tube Segment",
+			description = S("Crossing Pneumatic Tube Segment"),
 			inventory_image = "pipeworks_crossing_tube_inv.png",
 			plain = { "pipeworks_crossing_tube_plain.png" },
 			noctr = { "pipeworks_crossing_tube_noctr.png" },
@@ -138,7 +165,7 @@ end
 
 if pipeworks.enable_one_way_tube then
 	minetest.register_node("pipeworks:one_way_tube", {
-		description = "One way tube",
+		description = S("One way tube"),
 		tiles = {"pipeworks_one_way_tube_top.png", "pipeworks_one_way_tube_top.png", "pipeworks_one_way_tube_output.png",
 			"pipeworks_one_way_tube_input.png", "pipeworks_one_way_tube_side.png", "pipeworks_one_way_tube_top.png"},
 		paramtype2 = "facedir",
@@ -165,6 +192,7 @@ if pipeworks.enable_one_way_tube then
 		check_for_pole = pipeworks.check_for_vert_tube,
 		check_for_horiz_pole = pipeworks.check_for_horiz_tube
 	})
+	pipeworks.ui_cat_tube_list[#pipeworks.ui_cat_tube_list+1] = "pipeworks:one_way_tube"
 	minetest.register_craft({
 		output = "pipeworks:one_way_tube 2",
 		recipe = {
